import 'package:camera/camera.dart';
import 'package:flutter/material.dart';
import 'package:wakelock/wakelock.dart';

class CameraViewer extends StatefulWidget {
  final Widget widget;
  final Function(CameraImage) onImageStreamed;

  const CameraViewer(
      {super.key, required this.onImageStreamed, required this.widget});

  @override
  State<StatefulWidget> createState() {
    return CameraViewerState();
  }
}

class CameraViewerState extends State<CameraViewer> {
  List<CameraDescription> cameras = [];
  CameraController? cameraController;

  @override
  void initState() {
    super.initState();
    initCamera();
  }

  Future<void> initCamera() async {
    cameras = await availableCameras();
    cameraController = CameraController(cameras.first, ResolutionPreset.high,
        imageFormatGroup: ImageFormatGroup.yuv420, enableAudio: false);

    await cameraController!.initialize();

    if (cameraController!.value.isInitialized) {
      setState(() {});
    }

    await Wakelock.enable();

    cameraController!.setFlashMode(FlashMode.off);
    cameraController!.startImageStream(widget.onImageStreamed);
  }

  @override
  void dispose() {
    cameraController!.stopImageStream();
    cameraController!.dispose();
    Wakelock.disable();
    super.dispose();
  }

  void toggleFlashMode() {
    if (cameraController == null) {
      return;
    }

    if (cameraController!.value.flashMode == FlashMode.off) {
      cameraController!.setFlashMode(FlashMode.torch);
      setState(() {});
    }

    if (cameraController!.value.flashMode == FlashMode.torch) {
      cameraController!.setFlashMode(FlashMode.off);
      setState(() {});
    }
  }

  @override
  Widget build(BuildContext context) {
    final Size screenSize = MediaQuery.of(context).size;

    if (cameraController == null) {
      return Container();
    }

    return SafeArea(
      child: SizedBox(
        height: screenSize.height,
        width: screenSize.width,
        child: CameraPreview(
          cameraController!,
          child: Stack(
            children: [
              widget.widget,
              Align(
                alignment: Alignment.topRight,
                child: Padding(
                  padding: const EdgeInsets.all(8.0),
                  child: IconButton(
                      onPressed: toggleFlashMode,
                      icon: cameraController!.value.flashMode != FlashMode.off
                          ? const Icon(
                              Icons.flash_off,
                              color: Colors.white,
                            )
                          : const Icon(
                              Icons.flash_on_outlined,
                              color: Colors.orange,
                            )),
                ),
              )
            ],
          ),
        ),
      ),
    );
  }
}
