import 'package:flutter/material.dart';

class InterestZone extends CustomPainter {
  final double heightInterestFactor;
  final double widthInterestFactor;
  final Color borderColors;
  InterestZone(
      {required this.heightInterestFactor,
      required this.widthInterestFactor,
      required this.borderColors});

  @override
  void paint(Canvas canvas, Size size) {
    canvas.saveLayer(Rect.largest, Paint());
    final rect = Rect.fromLTWH(0, 0, size.width, size.height);

    final paint = Paint()
      ..color = Colors.black.withOpacity(0.5)
      ..style = PaintingStyle.fill;

    canvas.drawRect(rect, paint);
    final holeSize = Size(
        size.width * widthInterestFactor, size.height * heightInterestFactor);
    final holeRect = Rect.fromCenter(
      center: size.center(Offset.zero),
      width: holeSize.width,
      height: holeSize.height,
    );

    final RRect roundedRect =
        RRect.fromRectAndRadius(holeRect, const Radius.circular(20));

    final holePaint = Paint()
      ..blendMode = BlendMode.clear
      ..color = Colors.transparent
      ..strokeWidth = 2;

    final borderpaint = Paint()
      ..color = borderColors
      ..style = PaintingStyle.stroke
      ..strokeWidth = 2;

    canvas.drawRRect(roundedRect, holePaint);
    canvas.drawRRect(roundedRect, borderpaint);
    canvas.restore();
  }

  @override
  bool shouldRepaint(covariant CustomPainter oldDelegate) {
    return true;
  }
}
